using Base.Cartesian

"""
ndgrid(x...)

Calculate a Cartesian grid in an n-dimensional space from input vectors x_1, ..., x_n.

"""

function ndgrid(x::Vararg{Any,N}) where {N}
    T = eltype.(x)
    dims = map(length, x)
    X = ntuple(k -> Array{T[k],N}(undef, dims), Val(N))
    _ndgrid!(X, x)

    return X
end


macro refk(k, A::Symbol, ex)
    vars = Base.Cartesian.inlineanonymous(ex, k)
    Expr(:escape, Expr(:ref, A, vars))
end

@generated function _ndgrid!(dest::NTuple{N}, src::NTuple{N}) where {N}
    quote
        @nexprs $N k -> begin
            x = dest[k]
            y = src[k]
            @nloops $N i x begin
                @inbounds (@nref $N x i) = (@refk k y i)
            end
        end
    end
end